@@ -1,11 +1,14 @@
-import random
+"""Algorithms related to connected components of a hypergraph."""
+import xgi
 from xgi.exception import XGIError
 
 __all__ = [
     "is_connected",
     "connected_components",
+    "largest_connected_component",
     "number_connected_components",
     "node_connected_component",
+    "largest_connected_hypergraph",
 ]
 
 
@@ -21,23 +24,24 @@ def is_connected(H):
     Returns
     -------
     bool
-        Specifies whether the hypergraph is connected.
+        Whether the hypergraph is connected.
 
     See Also
     --------
     connected_components
     number_connected_components
+    largest_connected_component
+    largest_connected_hypergraph
 
     Example
     -------
-        >>> import xgi
-        >>> n = 1000
-        >>> m = n
-        >>> p = 0.01
-        >>> H = xgi.erdos_renyi_hypergraph(n, m, p)
-        >>> print(xgi.is_connected(H))
+    >>> import xgi
+    >>> H = xgi.random_hypergraph(10, [0.5, 0.01], seed=1)
+    >>> print(xgi.is_connected(H))
+    True
+
     """
-    return len(_plain_bfs(H, random.choice(list(H.nodes)))) == len(H)
+    return len(_plain_bfs(H, list(H.nodes)[0])) == len(H)
 
 
 def connected_components(H):
@@ -58,15 +62,16 @@ def connected_components(H):
     --------
     is_connected
     number_connected_components
+    largest_connected_component
+    largest_connected_hypergraph
 
     Example
     -------
-        >>> import xgi
-        >>> n = 1000
-        >>> m = n
-        >>> p = 0.01
-        >>> H = xgi.erdos_renyi_hypergraph(n, m, p)
-        >>> print([len(component) for component in xgi.connected_components(H)])
+    >>> import xgi
+    >>> H = xgi.random_hypergraph(50, [0.1, 0.01], seed=1)
+    >>> print([len(component) for component in xgi.connected_components(H)])
+    [50]
+
     """
     seen = set()
     for v in H:
@@ -88,21 +93,22 @@ def number_connected_components(H):
     Returns
     -------
     int
-        Returns the number of connected components of a hypergraph.
+        The number of connected components of the hypergraph.
 
     See Also
     --------
     is_connected
     connected_components
+    largest_connected_component
+    largest_connected_hypergraph
 
     Example
     -------
-        >>> import xgi
-        >>> n = 1000
-        >>> m = n
-        >>> p = 0.01
-        >>> H = xgi.erdos_renyi_hypergraph(n, m, p)
-        >>> print(xgi.number_connected_components(H))
+    >>> import xgi
+    >>> H = xgi.random_hypergraph(50, [0.1, 0.01], seed=1)
+    >>> print(xgi.number_connected_components(H))
+    1
+
     """
     num_cc = 0
     seen = set()
@@ -114,6 +120,38 @@ def number_connected_components(H):
     return num_cc
 
 
+def largest_connected_component(H):
+    """
+    A function to find the largest connected component of a hypergraph.
+
+    Parameters
+    ----------
+    H: Hypergraph object
+        The hypergraph of interest
+
+    Returns
+    -------
+    set
+        The largest connected component (a set of nodes) of the hypergraph.
+
+    See Also
+    --------
+    is_connected
+    connected_components
+    number_connected_components
+    largest_connected_hypergraph
+
+    Example
+    -------
+    >>> import xgi
+    >>> H = xgi.random_hypergraph(50, [0.1, 0.01], seed=1)
+    >>> print(xgi.number_connected_components(H))
+    1
+
+    """
+    return max(connected_components(H), key=len)
+
+
 def node_connected_component(H, n):
     """
     A function to find the connected component of which a node in the
@@ -138,12 +176,12 @@ def node_connected_component(H, n):
 
     Example
     -------
-        >>> import xgi
-        >>> n = 1000
-        >>> m = n
-        >>> p = 0.01
-        >>> H = xgi.erdos_renyi_hypergraph(n, m, p)
-        >>> print(xgi.node_connected_component(H, 0))
+    >>> import xgi
+    >>> H = xgi.random_hypergraph(50, [0.1, 0.01], seed=1)
+    >>> comp = xgi.node_connected_component(H, 0)
+    >>> print(type(comp), len(comp))
+    <class 'set'> 50
+
     """
     if n in H:
         return _plain_bfs(H, n)
@@ -163,3 +201,45 @@ def _plain_bfs(H, source):
                 seen.add(v)
                 nextlevel.update(H.neighbors(v))
     return seen
+
+
+def largest_connected_hypergraph(H, in_place=False):
+    """
+    A function to find the largest connected hypergraph from a data set.
+
+    Parameters
+    ----------
+    H: Hypergraph
+        The hypergraph of interest
+    in_place: bool, optional
+        If False, creates a copy; if True, modifies the existing hypergraph
+
+    Returns
+    -------
+    None
+        If in_place: modifies the existing hypergraph
+
+    xgi.Hypergraph
+        If not in_place: the hypergraph induced on the nodes of the
+        largest connected component.
+
+    See Also
+    --------
+    is_connected
+    connected_components
+    number_connected_components
+    largest_connected_component
+
+    Example
+    -------
+    >>> import xgi
+    >>> H = xgi.random_hypergraph(50, [0.1, 0.01], seed=1)
+    >>> print(xgi.number_connected_components(H))
+    1
+
+    """
+    connected_nodes = max(connected_components(H), key=len)
+    if not in_place:
+        return xgi.subhypergraph(H, nodes=connected_nodes).copy()
+    else:
+        H.remove_nodes_from(set(H.nodes).difference(connected_nodes))
