@@ -1,6 +1,4 @@
+from xgi.generators import classic, nonuniform, uniform
 from xgi.generators.classic import *
-from xgi.generators import classic
 from xgi.generators.nonuniform import *
-from xgi.generators import nonuniform
 from xgi.generators.uniform import *
-from xgi.generators import uniform
