@@ -1,83 +1,27 @@
+"""Generate random (non-uniform) hypergraphs."""
 import math
-import random
 import warnings
 from collections import defaultdict
 from itertools import combinations
-import numpy as np
-import xgi
 
+import networkx as nx
+import numpy as np
+
+import xgi
+from xgi.utils import py_random_state
 
 __all__ = [
-    "erdos_renyi_hypergraph",
     "chung_lu_hypergraph",
     "dcsbm_hypergraph",
     "random_hypergraph",
+    "random_simplicial_complex",
+    "random_flag_complex_d2",
+    "random_flag_complex",
 ]
 
 
-def erdos_renyi_hypergraph(n, m, p):
-    """A function to generate an Erdos-Renyi hypergraph
-
-    Parameters
-    ----------
-    n: int
-        Number of nodes
-    m: int
-        Number of edges
-    p: float
-        The probability that a bipartite edge is created
-
-    Returns
-    -------
-    Hypergraph object
-        The generated hypergraph
-
-    References
-    ----------
-    Implemented by Mirah Shi in HyperNetX and described for
-    bipartite networks by Aksoy et al. in https://doi.org/10.1093/comnet/cnx001
-
-    Examples
-    --------
-    >>> import xgi
-    >>> n = 1000
-    >>> m = n
-    >>> p = 0.01
-    >>> H = xgi.erdos_renyi_hypergraph(n, m, p)
-    """
-
-    H = xgi.empty_hypergraph()
-    H.add_nodes_from(range(n))
-
-    if p < 0.0 or p > 1.0:
-        raise ValueError("Invalid p value.")
-
-    if p == 0.0:
-        H = xgi.empty_hypergraph()
-        H.add_nodes_from(range(n))
-        return H
-
-    # this corresponds to a completely filled incidence matrix,
-    # not a complete hypergraph.
-    if p == 1.0:
-        H = xgi.empty_hypergraph()
-        H.add_edges_from([range(n) for i in range(m)])
-        return H
-
-    for u in range(n):
-        v = 0
-        while v < m:
-            # identify next pair
-            r = random.random()
-            v = v + math.floor(math.log(r) / math.log(1 - p))
-            if v < m:
-                # add vertex hyperedge pair
-                H.add_node_to_edge(v, u)
-            v = v + 1
-    return H
-
-
-def chung_lu_hypergraph(k1, k2):
+@py_random_state("seed")
+def chung_lu_hypergraph(k1, k2, seed=None):
     """A function to generate a Chung-Lu hypergraph
 
     Parameters
@@ -88,6 +32,8 @@ def chung_lu_hypergraph(k1, k2):
     k2 : dictionary
         Dictionary where the keys are edge ids
         and the values are edge sizes.
+    seed : integer, random_state, or None (default)
+            Indicator of random number generation state.
 
     Returns
     -------
@@ -118,11 +64,11 @@ def chung_lu_hypergraph(k1, k2):
     >>> k1 = {i : random.randint(1, 100) for i in range(n)}
     >>> k2 = {i : sorted(k1.values())[i] for i in range(n)}
     >>> H = xgi.chung_lu_hypergraph(k1, k2)
-    """
 
+    """
     # sort dictionary by degree in decreasing order
-    Nlabels = [n for n, _ in sorted(k1.items(), key=lambda d: d[1], reverse=True)]
-    Mlabels = [m for m, _ in sorted(k2.items(), key=lambda d: d[1], reverse=True)]
+    node_labels = [n for n, _ in sorted(k1.items(), key=lambda d: d[1], reverse=True)]
+    edge_labels = [m for m, _ in sorted(k2.items(), key=lambda d: d[1], reverse=True)]
 
     m = len(k2)
 
@@ -134,46 +80,51 @@ def chung_lu_hypergraph(k1, k2):
     S = sum(k1.values())
 
     H = xgi.empty_hypergraph()
-    H.add_nodes_from(Nlabels)
+    H.add_nodes_from(node_labels)
 
-    for u in Nlabels:
+    for u in node_labels:
         j = 0
-        v = Mlabels[j]  # start from beginning every time
+        v = edge_labels[j]  # start from beginning every time
         p = min((k1[u] * k2[v]) / S, 1)
 
         while j < m:
             if p != 1:
-                r = random.random()
-                j = j + math.floor(math.log(r) / math.log(1 - p))
+                r = seed.random()
+                try:
+                    j = j + math.floor(math.log(r) / math.log(1 - p))
+                except ZeroDivisionError:
+                    j = np.inf
+
             if j < m:
-                v = Mlabels[j]
+                v = edge_labels[j]
                 q = min((k1[u] * k2[v]) / S, 1)
-                r = random.random()
+                r = seed.random()
                 if r < q / p:
                     # no duplicates
-                    H.add_node_to_edge(u, v)
+                    H.add_node_to_edge(v, u)
                 p = q
                 j = j + 1
 
     return H
 
 
-def dcsbm_hypergraph(k1, k2, g1, g2, omega):
+@py_random_state("seed")
+def dcsbm_hypergraph(k1, k2, g1, g2, omega, seed=None):
     """A function to generate a DCSBM hypergraph.
 
     Parameters
     ----------
-    k1 : dictionary
+    k1 : dict
         This is a dictionary where the keys are node ids
         and the values are node degrees.
-    k2 : dictionary
+    k2 : dict
         This is a dictionary where the keys are edge ids
-        and the values are edge degrees also known as edge sizes.
-    g1 : dictionary
+        and the values are edge sizes.
+    g1 : dict
         This a dictionary where the keys are node ids
         and the values are the group ids to which the node belongs.
         The keys must match the keys of k1.
-    g2 : dictionary
+    g2 : dict
         This a dictionary where the keys are edge ids
         and the values are the group ids to which the edge belongs.
         The keys must match the keys of k2.
@@ -183,11 +134,12 @@ def dcsbm_hypergraph(k1, k2, g1, g2, omega):
         The number of rows must match the number of node communities
         and the number of columns must match the number of edge
         communities.
+    seed : int, random_state, or None (default)
+        Indicator of random number generation state.
 
     Returns
     -------
-    Hypergraph object
-        The generated hypergraph
+    xgi.Hypergraph
 
     Warns
     -----
@@ -211,21 +163,20 @@ def dcsbm_hypergraph(k1, k2, g1, g2, omega):
 
     Examples
     --------
-    >>> import xgi
-    >>> import random
-    >>> n = 100
-    >>> k1 = {i : random.randint(1, 100) for i in range(n)}
+    >>> import xgi; import random; import numpy as np
+    >>> n = 50
+    >>> k1 = {i : random.randint(1, n) for i in range(n)}
     >>> k2 = {i : sorted(k1.values())[i] for i in range(n)}
     >>> g1 = {i : random.choice([0, 1]) for i in range(n)}
     >>> g2 = {i : random.choice([0, 1]) for i in range(n)}
-    >>> omega = np.array([[100, 10], [10, 100]])
-    >>> H = xgi.dcsbm_hypergraph(k1, k2, g1, g2, omega)
+    >>> omega = np.array([[n//2, 10], [10, n//2]])
+    >>> # H = xgi.dcsbm_hypergraph(k1, k2, g1, g2, omega)
 
     """
 
     # sort dictionary by degree in decreasing order
-    Nlabels = [n for n, _ in sorted(k1.items(), key=lambda d: d[1], reverse=True)]
-    Mlabels = [m for m, _ in sorted(k2.items(), key=lambda d: d[1], reverse=True)]
+    node_labels = [n for n, _ in sorted(k1.items(), key=lambda d: d[1], reverse=True)]
+    edge_labels = [m for m, _ in sorted(k2.items(), key=lambda d: d[1], reverse=True)]
 
     # these checks verify that the sum of node and edge degrees and the sum of node degrees
     # and the sum of community connection matrix differ by less than a single edge.
@@ -240,18 +191,18 @@ def dcsbm_hypergraph(k1, k2, g1, g2, omega):
         )
 
     # get indices for each community
-    community1Indices = defaultdict(list)
-    for label in Nlabels:
+    community1_nodes = defaultdict(list)
+    for label in node_labels:
         group = g1[label]
-        community1Indices[group].append(label)
+        community1_nodes[group].append(label)
 
-    community2Indices = defaultdict(list)
-    for label in Mlabels:
+    community2_nodes = defaultdict(list)
+    for label in edge_labels:
         group = g2[label]
-        community2Indices[group].append(label)
+        community2_nodes[group].append(label)
 
     H = xgi.empty_hypergraph()
-    H.add_nodes_from(Nlabels)
+    H.add_nodes_from(node_labels)
 
     kappa1 = defaultdict(lambda: 0)
     kappa2 = defaultdict(lambda: 0)
@@ -260,41 +211,42 @@ def dcsbm_hypergraph(k1, k2, g1, g2, omega):
     for id, g in g2.items():
         kappa2[g] += k2[id]
 
-    for group1 in community1Indices.keys():
-        for group2 in community2Indices.keys():
+    for group1 in community1_nodes.keys():
+        for group2 in community2_nodes.keys():
             # for each constant probability patch
             try:
-                groupConstant = omega[group1, group2] / (
+                group_constant = omega[group1, group2] / (
                     kappa1[group1] * kappa2[group2]
                 )
-            except:
-                groupConstant = 0
+            except ZeroDivisionError:
+                group_constant = 0
 
-            for u in community1Indices[group1]:
+            for u in community1_nodes[group1]:
                 j = 0
-                v = community2Indices[group2][j]  # start from beginning every time
+                v = community2_nodes[group2][j]  # start from beginning every time
                 # max probability
-                p = min(k1[u] * k2[v] * groupConstant, 1)
-                while j < len(community2Indices[group2]):
+                p = min(k1[u] * k2[v] * group_constant, 1)
+                while j < len(community2_nodes[group2]):
                     if p != 1:
-                        r = random.random()
+                        r = seed.random()
                         try:
                             j = j + math.floor(math.log(r) / math.log(1 - p))
-                        except:
+                        except ZeroDivisionError:
                             j = np.inf
-                    if j < len(community2Indices[group2]):
-                        v = community2Indices[group2][j]
-                        q = min((k1[u] * k2[v]) * groupConstant, 1)
-                        r = random.random()
+                    if j < len(community2_nodes[group2]):
+                        v = community2_nodes[group2][j]
+                        q = min((k1[u] * k2[v]) * group_constant, 1)
+                        r = seed.random()
                         if r < q / p:
                             # no duplicates
-                            H.add_node_to_edge(u, v)
+                            H.add_node_to_edge(v, u)
                         p = q
                         j = j + 1
     return H
 
 
-def random_hypergraph(N, ps):
+@py_random_state("seed")
+def random_hypergraph(N, ps, seed=None):
     """Generates a random hypergraph
 
     Generate N nodes, and connect any d+1 nodes
@@ -309,6 +261,8 @@ def random_hypergraph(N, ps):
         hyperedge at each order d between any d+1 nodes. For example,
         ps[0] is the wiring probability of any edge (2 nodes), ps[1]
         of any triangles (3 nodes).
+    seed : integer, random_state, or None (default)
+            Indicator of random number generation state.
 
     Returns
     -------
@@ -322,9 +276,7 @@ def random_hypergraph(N, ps):
     Example
     -------
     >>> import xgi
-    >>> N = 100
-    >>> ps = [0.1, 0.01]
-    >>> H = xgi.random_hypergraph(N, ps)
+    >>> H = xgi.random_hypergraph(50, [0.1, 0.01])
 
     """
 
@@ -338,7 +290,7 @@ def random_hypergraph(N, ps):
         d = i + 1  # order, ps[0] is prob of edges (d=1)
 
         for hyperedge in combinations(nodes, d + 1):
-            if random.random() <= p:
+            if seed.random() <= p:
                 hyperedges.append(hyperedge)
 
     hyperedges += [[i] for i in nodes]  # add singleton edges
@@ -348,3 +300,146 @@ def random_hypergraph(N, ps):
     H.add_edges_from(hyperedges)
 
     return H
+
+
+@py_random_state("seed")
+def random_simplicial_complex(N, ps, seed=None):
+    """Generates a random hypergraph
+
+    Generate N nodes, and connect any d+1 nodes
+    by a simplex with probability ps[d-1]. For each simplex,
+    add all its subfaces if they do not already exist.
+
+    Parameters
+    ----------
+    N : int
+        Number of nodes
+    ps : list of float
+        List of probabilities (between 0 and 1) to create a
+        hyperedge at each order d between any d+1 nodes. For example,
+        ps[0] is the wiring probability of any edge (2 nodes), ps[1]
+        of any triangles (3 nodes).
+
+    Returns
+    -------
+    Simplicialcomplex object
+        The generated simplicial complex
+
+    References
+    ----------
+    Described as 'random simplicial complex' in
+    "Simplicial Models of Social Contagion", Nature Communications 10(1), 2485,
+    by I. Iacopini, G. Petri, A. Barrat & V. Latora (2019).
+    https://doi.org/10.1038/s41467-019-10431-6
+
+    Example
+    -------
+    >>> import xgi
+    >>> H = xgi.random_simplicial_complex(20, [0.1, 0.01])
+
+    """
+
+    if (np.any(np.array(ps) < 0)) or (np.any(np.array(ps) > 1)):
+        raise ValueError("All elements of ps must be between 0 and 1 included.")
+
+    nodes = range(N)
+    simplices = [[i] for i in nodes]  # add singleton edges
+
+    for i, p in enumerate(ps):
+        d = i + 1  # order, ps[0] is prob of edges (d=1)
+
+        for simplex in combinations(nodes, d + 1):
+            if seed.random() <= p:
+                simplices.append(simplex)
+
+    S = xgi.SimplicialComplex()
+    S.add_nodes_from(nodes)
+    S.add_simplices_from(simplices)
+
+    return S
+
+
+@py_random_state("seed")
+def random_flag_complex_d2(N, p, seed=None):
+    """Generate a maximal simplicial complex (up to order 2) from a
+    :math:`G_{N,p}` Erdős-Rényi random graph by filling all empty triangles with 2-simplices.
+
+    Parameters
+    ----------
+    N : int
+        Number of nodes
+    p : float
+        Probabilities (between 0 and 1) to create an edge
+        between any 2 nodes
+
+    Returns
+    -------
+    S : xgi.SimplicialComplex
+
+    Notes
+    -----
+    Computing all cliques quickly becomes heavy for large networks.
+    """
+
+    if (p < 0) or (p > 1):
+        raise ValueError("p must be between 0 and 1 included.")
+
+    G = nx.fast_gnp_random_graph(N, p, seed=seed)
+
+    nodes = G.nodes()
+    edges = list(G.edges())
+
+    # compute all triangles to fill
+    all_cliques = list(nx.enumerate_all_cliques(G))
+    triad_cliques = [tuple(x) for x in all_cliques if len(x) == 3]
+
+    simplices = edges + triad_cliques
+
+    S = xgi.SimplicialComplex()
+    S.add_nodes_from(nodes)
+    S.add_simplices_from(simplices)
+
+    return S
+
+
+@py_random_state(3)
+def random_flag_complex(N, p, max_order=2, seed=None):
+    """Generate a flag (or clique) complex from a
+    :math:`G_{N,p}` Erdős-Rényi random graph by filling all cliques up to dimension max_order.
+
+    Parameters
+    ----------
+    N : int
+        Number of nodes
+    p : float
+        Probabilities (between 0 and 1) to create an edge
+        between any 2 nodes
+
+    max_order : int
+        maximal dimension of simplices to add to the output simplicial complex
+
+    Returns
+    -------
+    S : xgi.SimplicialComplex
+
+    Notes
+    -----
+    Computing all cliques quickly becomes heavy for large networks.
+    """
+
+    if (p < 0) or (p > 1):
+        raise ValueError("p must be between 0 and 1 included.")
+
+    G = nx.fast_gnp_random_graph(N, p, seed=seed)
+
+    nodes = G.nodes()
+    edges = list(G.edges())
+
+    # compute all triangles to fill
+    max_cliques = list(nx.find_cliques(G))
+
+    S = xgi.SimplicialComplex()
+    S.add_nodes_from(nodes)
+    S.add_simplices_from(max_cliques, max_order=max_order)
+
+    return S
