@@ -1,6 +1,5 @@
-from xgi.readwrite.edgelist import *
-from xgi.readwrite import edgelist
-from xgi.readwrite.json import *
-from xgi.readwrite import json
+from xgi.readwrite import bipartite, edgelist, incidence, json
 from xgi.readwrite.bipartite import *
-from xgi.readwrite import bipartite
+from xgi.readwrite.edgelist import *
+from xgi.readwrite.incidence import *
+from xgi.readwrite.json import *
